"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const thenify = require("thenify");
const path_1 = require("path");
const raml_1_parser_1 = require("raml-1-parser");
const yargs = require("yargs");
const mkdrp = require("mkdirp");
const fs = require("fs");
const parseJson = require("parse-json");
const mkdirp = thenify(mkdrp);
const readFile = thenify(fs.readFile);
const writeFile = thenify(fs.writeFile);
function bin(generator, pkg, argv) {
    const cwd = process.cwd();
    const args = yargs
        .usage(`${pkg.description}\n\n$0 api.raml --out [directory]`)
        .version(pkg.version, 'version')
        .demand('o')
        .alias('o', 'out')
        .describe('o', 'Out directory')
        .alias('d', 'data')
        .describe('d', 'Path to JSON configuration file')
        .array('include')
        .alias('i', 'include')
        .describe('i', 'Include additional RAML files (E.g. extensions)')
        .parse(argv);
    return raml_1_parser_1.loadApi(args._[2], args.include || [], { rejectOnErrors: true })
        .then(function (api) {
        const json = api.expand(true).toJSON();
        if (args.data == null) {
            return Promise.resolve(generator(json));
        }
        const path = path_1.resolve(cwd, args.data);
        return readFile(path, 'utf8')
            .then((contents) => parseJson(contents, null, path))
            .then((data) => generator(json, data));
    })
        .then(function (output) {
        return objectToFs(path_1.resolve(cwd, args.out), output.files);
    })
        .then(function () {
        process.exit(0);
    })
        .catch(function (err) {
        if (err.parserErrors) {
            err.parserErrors.forEach((parserError) => {
                console.error(`${parserError.path} (${parserError.line + 1}, ${parserError.column + 1}): ${parserError.message}`);
            });
        }
        console.error(err.stack || err.message || err);
        process.exit(1);
    });
}
exports.bin = bin;
function objectToFs(path, object) {
    return Object.keys(object).reduce(function (promise, file) {
        const content = object[file];
        const filename = path_1.resolve(path, file);
        const output = path_1.dirname(filename);
        return promise
            .then(function () {
            return mkdirp(output);
        })
            .then(function () {
            return writeFile(filename, content);
        });
    }, mkdirp(path).then(() => undefined));
}
//# sourceMappingURL=bin.js.map